import os
#import os.path
#import os.readlink
from dataclasses import dataclass
from ..log import Log


@dataclass
class AinMap:
    path_raw: str
    path_scale: str


class AinOperator():
    AIN_PORT_MIN = 1
    AIN_PORT_UNIT = 4
    AIN_PORT_MAX = 16
    AIN_BOARD_MIN = 1
    AIN_BOARD_MAX = 4
    DEV_PATH_REF = '/sys/bus/iio/devices/iio:device'
    DEV_NAME = 'ads1015'

    def __init__(self):
        self._map = {}
        self._create_map()

    def _create_map(self):
        for i in range(0, 64):  # 64 is somehow
            board = -1
            if os.path.isdir(self.DEV_PATH_REF+str(i)):
                try:
                    with open(self.DEV_PATH_REF+str(i)+'/name') as f:
                        name = f.read()
                        if self.DEV_NAME in name:
                            reg = os.readlink(self.DEV_PATH_REF+str(i)+'/of_node')
                            if reg.endswith('49'):
                                board = 1
                            elif reg.endswith('48'):
                                board = 2
                            elif reg.endswith('4B') or reg.endswith('4b'):
                                board = 3
                            elif reg.endswith('4A') or reg.endswith('4a'):
                                board = 4
                except FileNotFoundError:
                    break
            else:
                break

            if board != -1:
                self._map_port(board, i)

    def _map_port(self, board, found):
        for j in range(self.AIN_PORT_MIN, self.AIN_PORT_UNIT+1):
            map = AinMap(path_raw=self.DEV_PATH_REF+str(found)+'/in_voltage'+str(j-1)+'_raw',
                         path_scale=self.DEV_PATH_REF+str(found)+'/in_voltage'+str(j-1)+'_scale')
            self._map[(board-1)*self.AIN_PORT_UNIT+j-1] = map

    def _port_to_map(self, port):
        return self._map[port-1]

    def value(self, port):
        log = Log.instance()
        raw = 0.0
        scale = 1.0

        if (port - 1) not in self._map.keys():
            log.warning('AinOperator::value: port:{0} is not found'.format(port))
            return None

        try:
            with open(self._port_to_map(port).path_raw, "r") as f:
                raw = int(f.read())
        except FileNotFoundError:
            log.error('AinOperator::value: file not found port:{0}'.format(port))
            return None

        try:
            with open(self._port_to_map(port).path_scale, "r") as f:
                scale = float(f.read())
        except FileNotFoundError:
            log.error('AinOperator::value: file not found port:{0}'.format(port))
            return None

        if scale == 0.0:
            return 0

        return int(raw * scale)

    def set_scale(self, port, scale):
        log = Log.instance()
        if scale not in (0.1875, 0.125, 0.0625, 0.03125, 0.015625, 0.007813):
            return
        try:
            with open(self._port_to_map(port).path_scale, "w") as f:
                 f.write(float(scale))
        except FileNotFoundError:
            log.error('AinOperator::set_scale: file not found port:{port}')
            return None

#
# End of File
#
