import os.path
from dataclasses import dataclass
from ..log import Log


@dataclass
class VinMap:
    path_raw: str
    path_scale: str


class VinOperator():
    DEV_PATH_REF = '/sys/bus/iio/devices/iio:device'
    DEV_NAME = '2198000.adc'
    R1 = 910
    R2 = 110

    def __init__(self):
        self._map = None
        self._create_map()

    def _create_map(self):
        for i in range(0, 64):  # 64 is somehow
            if os.path.isdir(self.DEV_PATH_REF+str(i)):
                try:
                    with open(self.DEV_PATH_REF+str(i)+'/name') as f:
                        name = f.read()
                        if self.DEV_NAME in name:
                            self._map = VinMap(path_raw=self.DEV_PATH_REF+str(i)+'/in_voltage1_raw',
                                               path_scale=self.DEV_PATH_REF+str(i)+'/in_voltage_scale')
                except FileNotFoundError:
                    break

    def value(self):
        log = Log.instance()
        raw = 0
        scale = 1.0

        if self._map is None:
            return None

        try:
            with open(self._map.path_raw, "r") as f:
                raw = int(f.read())
        except FileNotFoundError:
            log.error('VinOperator::raw: file not found')
            return None

        try:
            with open(self._map.path_scale, "r") as f:
                scale = float(f.read())
        except FileNotFoundError:
            log.error('VinOperator::scale: file not found')
            return None

        return int(((self.R1 + self.R2) * raw * scale + self.R2 / 2) / self.R2)

#
# End of File
#
